@@ -13,8 +13,8 @@ use Doctrine\ORM\Query\ParserResult;
 use Doctrine\ORM\Query\ResultSetMapping;
 use Doctrine\Tests\OrmFunctionalTestCase;
 use Generator;
+use PHPUnit\Framework\Attributes\DataProvider;
 use ReflectionMethod;
-use ReflectionProperty;
 use Symfony\Component\VarExporter\Instantiator;
 use Symfony\Component\VarExporter\VarExporter;
 
@@ -32,11 +32,8 @@ class ParserResultSerializationTest extends OrmFunctionalTestCase
         parent::setUp();
     }
 
-    /**
-     * @param Closure(ParserResult): ParserResult $toSerializedAndBack
-     *
-     * @dataProvider provideToSerializedAndBack
-     */
+    /** @param Closure(ParserResult): ParserResult $toSerializedAndBack */
+    #[DataProvider('provideToSerializedAndBack')]
     public function testSerializeParserResultForQueryWithSqlWalker(Closure $toSerializedAndBack): void
     {
         $query = $this->_em
@@ -54,11 +51,8 @@ class ParserResultSerializationTest extends OrmFunctionalTestCase
         $this->assertNotNull($unserialized->prepareSqlExecutor($query));
     }
 
-    /**
-     * @param Closure(ParserResult): ParserResult $toSerializedAndBack
-     *
-     * @dataProvider provideToSerializedAndBack
-     */
+    /** @param Closure(ParserResult): ParserResult $toSerializedAndBack */
+    #[DataProvider('provideToSerializedAndBack')]
     public function testSerializeParserResultForQueryWithSqlOutputWalker(Closure $toSerializedAndBack): void
     {
         $query = $this->_em
@@ -74,7 +68,7 @@ class ParserResultSerializationTest extends OrmFunctionalTestCase
     }
 
     /** @return Generator<string, array{Closure(ParserResult): ParserResult}> */
-    public function provideToSerializedAndBack(): Generator
+    public static function provideToSerializedAndBack(): Generator
     {
         yield 'native serialization function' => [
             static function (ParserResult $parserResult): ParserResult {
@@ -84,7 +78,7 @@ class ParserResultSerializationTest extends OrmFunctionalTestCase
 
         $instantiatorMethod = new ReflectionMethod(Instantiator::class, 'instantiate');
         if ($instantiatorMethod->getReturnType() === null) {
-            $this->markTestSkipped('symfony/var-exporter 5.4+ is required.');
+            self::markTestSkipped('symfony/var-exporter 5.4+ is required.');
         }
 
         yield 'symfony/var-exporter' => [
@@ -94,36 +88,7 @@ class ParserResultSerializationTest extends OrmFunctionalTestCase
         ];
     }
 
-    public function testItSerializesParserResultWithAForwardCompatibleFormat(): void
-    {
-        $query = $this->_em
-            ->createQuery('SELECT u FROM Doctrine\Tests\Models\Company\CompanyEmployee u WHERE u.name = :name');
-
-        // Use the (legacy) SqlWalker which directly puts an SqlExecutor instance into the parser result
-        $query->setHint(Query::HINT_CUSTOM_OUTPUT_WALKER, Query\SqlWalker::class);
-
-        $parserResult = self::parseQuery($query);
-        $serialized   = serialize($parserResult);
-        $this->assertStringNotContainsString(
-            '_sqlStatements',
-            $serialized,
-            'ParserResult should not contain any reference to _sqlStatements, which is a legacy property.'
-        );
-        $unserialized = unserialize($serialized);
-
-        $r = new ReflectionProperty($unserialized->getSqlExecutor(), '_sqlStatements');
-        $r->setAccessible(true);
-
-        $this->assertSame(
-            $r->getValue($unserialized->getSqlExecutor()),
-            $unserialized->getSqlExecutor()->getSqlStatements(),
-            'The legacy property should be populated with the same value as the new one.'
-        );
-    }
-
-    /**
-     * @dataProvider provideSerializedSingleSelectResults
-     */
+    #[DataProvider('provideSerializedSingleSelectResults')]
     public function testUnserializeSingleSelectResult(string $serialized): void
     {
         $unserialized = unserialize($serialized);
@@ -138,8 +103,6 @@ class ParserResultSerializationTest extends OrmFunctionalTestCase
     /** @return Generator<string, array{string}> */
     public static function provideSerializedSingleSelectResults(): Generator
     {
-        yield '2.14.3' => [rtrim(file_get_contents(__DIR__ . '/ParserResults/single_select_2_14_3.txt'), "\n")];
-        yield '2.15.0' => [rtrim(file_get_contents(__DIR__ . '/ParserResults/single_select_2_15_0.txt'), "\n")];
         yield '2.17.0' => [rtrim(file_get_contents(__DIR__ . '/ParserResults/single_select_2_17_0.txt'), "\n")];
     }
 
